﻿using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Interactivity;
using Telerik.Windows.Controls;

namespace CheckBoxColumn_SelectAll
{
	public class SelectItemsBehavior : Behavior<RadGridView>
	{
		private bool IsSelecting { get; set; }

		protected override void OnAttached()
		{
			base.OnAttached();

			this.AssociatedObject.SelectionChanged += OnSelectionChanged;
			this.AssociatedObject.Items.CollectionChanged += Items_CollectionChanged;

			StartSyncing();
		}

		private void StartSyncing()
		{
			this.StartSyncing(this.AssociatedObject.Items.OfType<INotifyPropertyChanged>());
		}

		private void StopSyncing()
		{
			this.StopSyncing(this.AssociatedObject.Items.OfType<INotifyPropertyChanged>());
		}

		void Items_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			switch (e.Action)
			{
				case NotifyCollectionChangedAction.Add:
					this.StartSyncing(e.NewItems.OfType<INotifyPropertyChanged>());
					break;
				case NotifyCollectionChangedAction.Remove:
					this.StopSyncing(e.OldItems.OfType<INotifyPropertyChanged>());
					break;
				case NotifyCollectionChangedAction.Replace:
					this.StartSyncing(e.NewItems.OfType<INotifyPropertyChanged>());
					this.StopSyncing(e.OldItems.OfType<INotifyPropertyChanged>());
					break;
				case NotifyCollectionChangedAction.Reset:
					this.StartSyncing();
					break;
				default:
					break;
			}			
		}

		private void StartSyncing(IEnumerable<INotifyPropertyChanged> items)
		{
			var firstItem = items.FirstOrDefault();
			if (firstItem != null)
			{
				var selectionPropertyInfo = firstItem.GetType().GetProperty(this.SelectionSynchronizationPropertyName);

				foreach (var item in items)
				{
					if ((bool)selectionPropertyInfo.GetValue(item, null))
					{
						this.AssociatedObject.SelectedItems.Add(item);
					}

					item.PropertyChanged += OnItemPropertyChanged;
				}
			}	
		}

		private void StopSyncing(IEnumerable<INotifyPropertyChanged> items)
		{
			foreach (var item in items)
			{
				item.PropertyChanged -= OnItemPropertyChanged;
			}
		}

		void OnItemPropertyChanged(object sender, PropertyChangedEventArgs e)
		{
			if (this.IsSelecting)
			{
				return;
			}

			if (e.PropertyName == this.SelectionSynchronizationPropertyName)
			{
				var isRemoved = this.AssociatedObject.SelectedItems.Remove(sender);
				if (!isRemoved)
				{
					this.AssociatedObject.SelectedItems.Add(sender);
				}
			}
		}

		protected override void OnDetaching()
		{
			base.OnDetaching();

			this.AssociatedObject.SelectionChanged -= OnSelectionChanged;
			this.StopSyncing();
			
		}

		public string SelectionSynchronizationPropertyName
		{
			get 
			{
				return (string)GetValue(SelectionSynchronizationPropertyNameProperty); 
			}
			set 
			{
				SetValue(SelectionSynchronizationPropertyNameProperty, value); 
			}
		}

		public static readonly DependencyProperty SelectionSynchronizationPropertyNameProperty =
		DependencyProperty.Register("SelectionSynchronizationPropertyName", typeof(string), typeof(SelectItemsBehavior), new PropertyMetadata(null));

		private void OnSelectionChanged(object sender, Telerik.Windows.Controls.SelectionChangeEventArgs e)
		{
			object representativeItem = GetRepresentativeItem(e);

			var selectionPropertyInfo = representativeItem.GetType().GetProperty(this.SelectionSynchronizationPropertyName);
			if (selectionPropertyInfo != null)
			{
				this.IsSelecting = true;

				foreach (var item in e.AddedItems)
				{
					selectionPropertyInfo.SetValue(item, true, null);
				}

				foreach (var item in e.RemovedItems)
				{
					selectionPropertyInfo.SetValue(item, false, null);
				}

				this.IsSelecting = false;
			}
		}

		private static object GetRepresentativeItem(SelectionChangeEventArgs e)
		{
			if (e.AddedItems.Count > 0)
			{
				return e.AddedItems[0];
			}

			if (e.RemovedItems.Count > 0)
			{
				return e.RemovedItems[0];
			}

			return null;
		}
	}

}
